# !/usr/bin/env python
# coding=utf-8

import time
import sys,os
import threading
import math
import struct, array, time, json
from mavlink import *
from socket import *
from pynput import keyboard

HOST_IP = "192.168.4.1"  # 无人机Wi-Fi数传模块 Mlink-esp 的IP
HOST_PORT = 333

class datalink :
	def __init__(self):
		self.socket_tcp=socket(AF_INET,SOCK_STREAM)
		self.host_addr = (HOST_IP, HOST_PORT)
		self.f=None
		self.mav_drone=None
		self.message_drone=None
		self.msg_id=None
		self.x=0 # m
		self.y=0
		self.z=0
		self.vx=0 # m/s
		self.vy=0
		self.vz=0
		self.afx=0 # m/ss
		self.afy=0
		self.afz=0
		self.yaw=0 # rad
		self.yaw_rate=0 # rad/s
		self.pos_x=0 # m
		self.pos_y=0
		self.pos_z=0
		self.att_roll=0 # rad
		self.att_pitch=0
		self.att_yaw=0
		self.relative_alt=0 # m
		self.batt_voltage=0 # V
		self.batt_current=0 # A
		self.init_xy=False

	def set_arm(self):
		self.mav_drone.set_mode_send(0, MAV_MODE_AUTO_ARMED, 0)

	def set_disarm(self):
		self.mav_drone.set_mode_send(0, MAV_MODE_AUTO_DISARMED, 0)

	def set_takeoff(self):
		self.mav_drone.command_long_send(0, 0, MAV_CMD_NAV_TAKEOFF, 0, 0, 0, 0, 0, 0, 0, 0)

	def set_land(self):
		self.mav_drone.command_long_send(0, 0, MAV_CMD_NAV_LAND, 0, 0, 0, 0, 0, 0, 0, 0)

	# dx:m  dy:m  d_alt:m  d_yaw:rad
	def set_pose(self, dx, dy, d_alt, d_yaw):
		if (self.pos_x != 0) and (self.pos_y != 0) and (self.att_yaw != 0):
			global_dx = dx * math.cos(self.att_yaw) - dy * math.sin(self.att_yaw)
			global_dy = dx * math.sin(self.att_yaw) + dy * math.cos(self.att_yaw)
			self.x = self.pos_x + global_dx
			self.y = self.pos_y + global_dy
			self.z = self.relative_alt + d_alt
			self.yaw = self.att_yaw + d_yaw
			self.mav_drone.set_position_target_local_ned_send(0, 0, 0, 0, 0, self.x, self.y, self.z, 0, 0, 0, 0, 0, 0, self.yaw, 0)

	# dx:m  dy:m  yaw:rad
	def set_xy_pose(self, dx, dy, yaw):
		if (self.pos_x != 0) and (self.pos_y != 0) and (self.att_yaw != 0):
			global_dx = dx * math.cos(self.att_yaw) - dy * math.sin(self.att_yaw)
			global_dy = dx * math.sin(self.att_yaw) + dy * math.cos(self.att_yaw)
			self.x = self.pos_x + global_dx
			self.y = self.pos_y + global_dy
			self.z = 0
			self.yaw = yaw
			self.mav_drone.set_position_target_local_ned_send(0, 0, 0, 0, 0, self.x, self.y, self.z, 0, 0, 0, 0, 0, 0, self.yaw, 0)

	# roll:rad  pitch:rad  yaw:rad  alt:m
	def set_att_alt(self, roll, pitch, yaw, alt):
		self.afx = roll
		self.afy = pitch
		self.yaw = yaw
		self.z = alt
		self.mav_drone.set_position_target_local_ned_send(0, 0, 0, MAV_FRAME_BODY_NED, 0, 0, 0, self.z, 0, 0, 0, self.afx, self.afy, 0, self.yaw, 0)

	class f_drone(object):
		def __init__(self, file):
			self.file=file
			self.buf =[]
		def write(self,data):
			self.file.send(data)

	def drone(self):
		self.f=self.f_drone(self.socket_tcp)
		self.mav_drone = MAVLink(self.f)
		while True:
			try:
				c = self.socket_tcp.recv(1)
			except:
				pass
				continue
			if c!=None :
				try:
					self.message_drone = self.mav_drone.parse_char(c)
					if self.message_drone != None:
						self.msg_id = self.message_drone.get_msgId()
						if self.msg_id == MAVLINK_MSG_ID_HEARTBEAT:
							print('heartbeat\n')
						elif self.msg_id == MAVLINK_MSG_ID_GLOBAL_VISION_POSITION_ESTIMATE:
							if(isinstance(self.message_drone, MAVLink_global_vision_position_estimate_message)):
								self.pos_x=self.message_drone.x/100
								self.pos_y=self.message_drone.y/100
								self.pos_z=self.message_drone.z/100
								self.att_roll=self.message_drone.roll
								self.att_pitch=self.message_drone.pitch
								self.att_yaw=self.message_drone.yaw
								#print('pos:',self.pos_x,'|',self.pos_y,'|',self.pos_z,'\n')
						elif self.msg_id == MAVLINK_MSG_ID_GLOBAL_POSITION_INT:
							if(isinstance(self.message_drone, MAVLink_global_position_int_message)):
								self.relative_alt=float(self.message_drone.relative_alt)/1000
								#print('relative_alt:',self.relative_alt,'\n')
						elif self.msg_id == MAVLINK_MSG_ID_BATTERY_STATUS:
							if(isinstance(self.message_drone, MAVLink_battery_status_message)):
								self.batt_voltage=float(self.message_drone.voltages[1])/1000
								self.batt_current=float(self.message_drone.current_battery)/100
								print('batt:',self.batt_voltage,'V|',self.batt_current,'A\n')
						#else:
							#print('msg:',self.msg_id,'\n')
				except Exception as e:
					self.message_drone = None
					#print("Error parsing message:", e)

dl=datalink()
def on_press(key):
	try:
		# 获取按下的按键
		print('You pressed {0}'.format(key.char))
		if key.char=='a' :
			dl.set_arm()
			print('解锁')
		if key.char=='d' :
			dl.set_disarm()
			print('锁定')
		if key.char=='t' :
			dl.set_takeoff()
			dl.set_xy_pose(0, 0, 0)
			print('起飞')
		if key.char=='l' :
			dl.set_land()
			print('降落')
	except AttributeError:
		print('You pressed {0}'.format(key))
		if format(key) == 'Key.up':
			dl.set_xy_pose(0.2, 0, 0)
			print('前进')
		if format(key) == 'Key.down':
			dl.set_xy_pose(-0.2, 0, 0)
			print('后退')
		if format(key) == 'Key.right':
			dl.set_xy_pose(0, 0.2, 0)
			print('右移')
		if format(key) == 'Key.left':
			dl.set_xy_pose(0, -0.2, 0)
			print('左移')

def main():
	print('data link is starting...\n')
	dl.socket_tcp.connect(dl.host_addr)
	drone_thread=threading.Thread(target=dl.drone)
	try:
		drone_thread.start()
		# 创建一个键盘监听器
		listener = keyboard.Listener(on_press=on_press)
		# 开启监听
		listener.start()
		while True:
			dl.mav_drone.heartbeat_send(MAV_TYPE_GCS, MAV_AUTOPILOT_INVALID, MAV_MODE_FLAG_MANUAL_INPUT_ENABLED, 0, MAV_STATE_STANDBY, 1)
			time.sleep(1)
	except Exception as e:
		print("Error sending message:", e)
	finally:
		drone_thread.join()
		dl.socket_tcp.close()
		listener.join()

if __name__ == '__main__':
	main()
